tilecollider = require 'tilecollider'
-------------------------------------------------------------------------------
--- V EXPERIMENT! V ---

	camera = {}
	camera.x = 0
	camera.y = 0
	camera.scaleX = 1
	camera.scaleY = 1
	camera.rotation = 0

	function camera:set()
	  love.graphics.push()
	  love.graphics.rotate(-self.rotation)
	  love.graphics.scale(1 / self.scaleX, 1 / self.scaleY)
	  love.graphics.translate(-self.x, -self.y)
	end

	function camera:unset()
	  love.graphics.pop()
	end

	function camera:move(dx, dy)
	  self.x = self.x + (dx or 0)
	  self.y = self.y + (dy or 0)
	end

	function camera:rotate(dr)
	  self.rotation = self.rotation + dr
	end

	function camera:scale(sx, sy)
	  sx = sx or 1
	  self.scaleX = self.scaleX * sx
	  self.scaleY = self.scaleY * (sy or sx)
	end

	function camera:setPosition(x, y)
	  self.x = x or self.x
	  self.y = y or self.y
	end

	function camera:setScale(sx, sy)
	  self.scaleX = sx or self.scaleX
	  self.scaleY = sy or self.scaleY
	end
-- An appension for this experiment.
--	CollidingAbove = false --moving up
--	CollidingBelow = false --moving down
--	CollidingLeftly = false --moving left
--	CollidingRightly = false --moving right
	
--========--	
	
-- create player
p1  = {x = 64, y = 64, w = 16, h = 16, Xspeed = 0, Yspeed = 0}
-- roughamount = 250

function p1:unpack()
	return self.x,self.y,self.w,self.h
end

function p1:draw()
	love.graphics.rectangle('line',self:unpack())
end

-------------------------------------------------------------------------------

-- our map
grid = 
	{
		tileWidth   = 32,
		tileHeight  = 32,
		
		{2,2,2,2,2,2,2,2,2,2,2,2,2},
		{2,1,1,1,1,1,1,1,1,1,1,1,2},
		{2,1,1,1,1,1,1,1,1,1,1,1,2},
		{2,1,1,1,1,20,1,1,1,1,'b',1,2},
		{2,1,1,1,1,1,1,1,1,1,1,1,2},
		{2,1,1,3,1,1,1,1,1,1,1,1,2},
		{2,1,1,1,1,1,1,1,1,'g',1,1,2},
		{2,1,1,1,1,1,1,1,1,1,1,1,2},
		{2,1,1,1,1,1,1,1,1,1,1,4,2},
		{2,1,1,1,1,1,1,1,1,1,4,2,2},
		{2,2,2,2,2,2,2,2,1,2,2,2,2},
		{2,1,1,1,1,1,1,1,1,1,1,1,2},
		{2,1,1,1,1,1,1,1,1,1,1,1,2},
		{2,1,1,1,1,4,1,1,1,1,1,1,2},
		{2,1,1,1,1,1,1,1,1,3,1,1,2},
		{2,1,1,1,1,1,1,1,1,1,1,1,2},
		{2,1,1,1,'g',1,1,1,1,1,1,1,2},
		{2,1,1,1,1,1,1,1,1,1,1,1,2},
		{2,1,1,1,1,1,1,1,2,1,1,1,2},
		{2,1,1,1,1,1,1,1,1,1,1,1,2},
		{2,2,2,2,2,2,2,2,2,2,2,2,2}
	}
	

-- some information about each tile
tileset = 
{
	[1] =
	{
		color = {255,255,255},
		type  = 1,
	},
	[2] =
	{
		color = {255,0,0},
		type  = 2,
	},
	g =
	{
		color = {0,255,0},
		type  = 3,
	},
	b =
	{
		color = {0,0,255},
		type  = 4,
	},
	-- vertical slope tile
	[3] =
	{
		color = {0,255,255},
		type  = 'vert',
	},
	-- horizontal slope tile
	[4] =
	{
		color = {255,100,0},
		type  = 'horz',
	},
	
	-- My attempted invention.
	[20] =
	{
		color = {255,178,60},
		type = 'newcurve',
		horizontal = {0,1,3,4},
		vertical = {0,3,3,0},
	},
}


-------------------------------------------------------------------------------

-- lookup table for slope tiles
heightmaps    = {}
heightmaps[3] = {     vertical = (function() local t = {} for i = 1,64 do t[i] = i end return t end)()}
heightmaps[4] = { horizontal = (function() local t = {} for i = 1,64 do t[i] = i end return t end)()}
heightmaps[20] = {horizontal = (function() local t = {} for i = 1,64 do t[i] = i end return t end)()}

-- callback for handler to get tile
function getTile(tx,ty)
	return grid[ty][tx]
end

-- collision callback for handler
function isResolvable(side,tile,gx,gy)
	local tileData = tileset[tile]
	local tileType = tileData.type

	if tileType == 2 then return true end
	
	if tileType == 'vert' and side == 'bottom' then return true end
	
	if tileType == 'horz' and side == 'right' then return true end
	
	-- one way tile
	if tileType == 3 then --- three = g/GREEN
		if side == 'left' and prevLeftGX > gx then return true end
	end
	
	-- one way tile
	if tileType == 4 then --- four = b/BLUE
		if side == 'right' and prevRightGX > gx then return true end
	end
	
	if tileType == 'newcurve' then return true end
end

-------------------------------------------------------------------------------

-- tile collision handler
handler = tilecollider(getTile, grid.tileWidth,grid.tileHeight, isResolvable,heightmaps)

-------------------------------------------------------------------------------

-- for one way tiles (see below)
function getTileRange(tw,th,x,y,w,h)
	gx,gy   = math.floor(x/tw)+1,math.floor(y/th)+1
	gx2,gy2 = math.ceil( (x+w)/tw ),math.ceil( (y+h)/th )
	return gx,gy,gx2,gy2
end

function love.update(dt)

	velocity = 400


	if love.keyboard.isDown('left') then
		dx = -velocity*dt
--		if CollidingLeftly == false then
--			camera:move(-0.44,0)
--		end		
	elseif love.keyboard.isDown('right') then
		dx = velocity*dt
--		if CollidingRightly == false then
--			camera:move(0.44,0)
--		end
	else 
		dx = 0
	end
	if love.keyboard.isDown('up') then
		dy = -velocity*dt
--		if CollidingAbove == false then
--			camera:move(0,-0.44)
--		end
	elseif love.keyboard.isDown('down') then
		dy = velocity*dt
--		if CollidingBelow == false then
--			camera:move(0,0.44)
--		end
	else
		dy = 0
	end
	
	camera.x = p1.x - 300
	camera.y = p1.y - 260

--[[	p1.x = p1.x + p1.Xspeed * dt
	p1.y = p1.y + p1.Yspeed * dt

	if love.keyboard.isDown('left') then
		p1.Xspeed = roughamount * -1
	elseif love.keyboard.isDown('right') then
		p1.Xspeed = roughamount * 1
	else 
		p1.Xspeed = 0
	end
	if love.keyboard.isDown('up') then
		p1.Yspeed = roughamount * -1
	elseif love.keyboard.isDown('down') then
		p1.Yspeed = roughamount * 1
	else
		p1.Yspeed = 0
	end
	
--]]
	
	prevLeftGX,_,prevRightGX = getTileRange(grid.tileWidth,grid.tileHeight,p1:unpack())
	
-- move and resolve collisions
	p1.x = p1.x+dx
	if dx > 0 then
		newx = handler:rightResolve(p1:unpack())
	elseif dx < 0 then
		newx = handler:leftResolve(p1:unpack())
	else
		newx = handler:rightResolve(p1:unpack())
		if newx == p1.x then newx = handler:leftResolve(p1:unpack()) end
	end
	
	p1.x = newx
	
	p1.y = p1.y+dy
	
	if dy > 0 then
		newy = handler:bottomResolve(p1:unpack())
	elseif dy < 0 then
		newy = handler:topResolve(p1:unpack())
	else
		newy = handler:bottomResolve(p1:unpack())
		if newy == p1.y then newy = handler:topResolve(p1:unpack()) end
	end
	
	p1.y = newy
end
-------------------------------------------------------------------------------
-- ===
function love.keypressed(key)

	if key == '1' then
		print(dx)
	end

	if key == '2' then
		print(dy)
	end

	if key == '3' then
		print("YX = " .. p1.Xspeed)
	end

	if key == '4' then
		print("YS = " .. p1.Yspeed)
	end
	
	if key == '5' then
		print(gx)
		print(gy)
	end
	
	if key == '6' then
		print(tx)
		print(ty)
	end

	if key == '-' then
		print("\n\n\n\n\n\n\n\n\n")
	end
end
-- ===
function love.draw()
	camera:set()
	local tw,th = grid.tileWidth,grid.tileHeight
	for ty,t in ipairs(grid) do
		ty = ty -- -1
		for tx,tile in ipairs(t) do
				tx = tx -- -1
				love.graphics.setColor(tileset[tile].color)
				if tile == 3 or tile == 4 or tile == 20 then
					-- draw slopes
					love.graphics.polygon('fill',tx*tw,(ty+1)*th,(tx+1)*tw,(ty+1)*th,(tx+1)*tw,(ty*th))
				else
					-- draw regular blocks
					love.graphics.rectangle('fill',tx*tw,ty*th,tw,th)
				end
		end
	end
	p1:draw('fill')
	camera:unset()

	love.graphics.setColor(0,22,0)
	love.graphics.rectangle("fill",0,480,800,100)

	love.graphics.setColor(0,240,240)
	love.graphics.print("The operation was a success.",100,490)
	
end